@@ -37,17 +37,21 @@ use std::{
 /// assert_eq!(printed, "01234567 89");
 /// ```
 pub fn print_bytes<W: Write>(mut slice: &[u8], writer: &mut W) -> std::fmt::Result {
-    while !slice.is_empty() {
-        let (chunk, remaining) = slice.split_at(4.min(slice.len()));
-        slice = remaining;
-        for byte in chunk {
-            write!(writer, "{:02x}", byte)?;
-        }
-        if !slice.is_empty() {
-            writer.write_char(' ')?;
+    if let Ok(as_str) = std::str::from_utf8(slice) {
+        write!(writer, "{as_str:?}")
+    } else {
+        while !slice.is_empty() {
+            let (chunk, remaining) = slice.split_at(4.min(slice.len()));
+            slice = remaining;
+            for byte in chunk {
+                write!(writer, "{:02x}", byte)?;
+            }
+            if !slice.is_empty() {
+                writer.write_char(' ')?;
+            }
         }
+        Ok(())
     }
-    Ok(())
 }
 
 /// An immutable buffer of bytes that can be cloned, sliced, and read into
@@ -93,6 +97,12 @@ impl<'a> Default for ArcBytes<'a> {
     }
 }
 
+impl<'a> AsRef<[u8]> for Bytes<'a> {
+    fn as_ref(&self) -> &[u8] {
+        &**self
+    }
+}
+
 #[test]
 fn default_is_new() {
     assert_eq!(ArcBytes::new(), ArcBytes::default());
@@ -124,47 +134,53 @@ impl<'a, 'b> PartialEq<ArcBytes<'b>> for ArcBytes<'a> {
     }
 }
 
-impl<'a> PartialEq<[u8]> for ArcBytes<'a> {
-    fn eq(&self, other: &[u8]) -> bool {
-        self.as_slice().cmp(other) == Ordering::Equal
-    }
+macro_rules! impl_std_cmp {
+    ($self:ty) => {
+        impl<'a> PartialEq<[u8]> for $self {
+            fn eq(&self, other: &[u8]) -> bool {
+                self.as_slice().cmp(other) == std::cmp::Ordering::Equal
+            }
+        }
+
+        impl<'a, const SIZE: usize> PartialEq<[u8; SIZE]> for $self {
+            fn eq(&self, other: &[u8; SIZE]) -> bool {
+                self.as_slice().cmp(other) == std::cmp::Ordering::Equal
+            }
+        }
+
+        impl<'a, 'b> PartialEq<&'b [u8]> for $self {
+            fn eq(&self, other: &&'b [u8]) -> bool {
+                self.as_slice().cmp(other) == std::cmp::Ordering::Equal
+            }
+        }
+
+        impl<'a> PartialOrd<[u8]> for $self {
+            fn partial_cmp(&self, other: &[u8]) -> Option<std::cmp::Ordering> {
+                self.as_slice().partial_cmp(other)
+            }
+        }
+
+        impl<'a, 'b, const SIZE: usize> PartialOrd<&'b [u8; SIZE]> for $self {
+            fn partial_cmp(&self, other: &&'b [u8; SIZE]) -> Option<std::cmp::Ordering> {
+                self.as_slice().partial_cmp(&other[..])
+            }
+        }
+
+        impl<'b, 'a> PartialOrd<&'b [u8]> for $self {
+            fn partial_cmp(&self, other: &&'b [u8]) -> Option<std::cmp::Ordering> {
+                self.as_slice().partial_cmp(other)
+            }
+        }
+
+        impl<'a, 'b, const N: usize> PartialEq<&'b [u8; N]> for $self {
+            fn eq(&self, other: &&'b [u8; N]) -> bool {
+                self.as_slice().cmp(*other) == std::cmp::Ordering::Equal
+            }
+        }
+    };
 }
 
-impl<'a, const SIZE: usize> PartialEq<[u8; SIZE]> for ArcBytes<'a> {
-    fn eq(&self, other: &[u8; SIZE]) -> bool {
-        self.as_slice().cmp(other) == Ordering::Equal
-    }
-}
-
-impl<'a, 'b> PartialEq<&'b [u8]> for ArcBytes<'a> {
-    fn eq(&self, other: &&'b [u8]) -> bool {
-        self.as_slice().cmp(other) == Ordering::Equal
-    }
-}
-
-impl<'a> PartialOrd<[u8]> for ArcBytes<'a> {
-    fn partial_cmp(&self, other: &[u8]) -> Option<Ordering> {
-        self.as_slice().partial_cmp(other)
-    }
-}
-
-impl<'a, 'b, const SIZE: usize> PartialOrd<&'b [u8; SIZE]> for ArcBytes<'a> {
-    fn partial_cmp(&self, other: &&'b [u8; SIZE]) -> Option<Ordering> {
-        self.as_slice().partial_cmp(&other[..])
-    }
-}
-
-impl<'b, 'a> PartialOrd<&'b [u8]> for ArcBytes<'a> {
-    fn partial_cmp(&self, other: &&'b [u8]) -> Option<Ordering> {
-        self.as_slice().partial_cmp(other)
-    }
-}
-
-impl<'a, 'b, const N: usize> PartialEq<&'b [u8; N]> for ArcBytes<'a> {
-    fn eq(&self, other: &&'b [u8; N]) -> bool {
-        self.as_slice().cmp(*other) == Ordering::Equal
-    }
-}
+impl_std_cmp!(ArcBytes<'a>);
 
 impl<'a> Ord for ArcBytes<'a> {
     fn cmp(&self, other: &Self) -> Ordering {
@@ -511,6 +527,12 @@ impl<'a> std::borrow::Borrow<[u8]> for ArcBytes<'a> {
     }
 }
 
+impl<'a> AsRef<[u8]> for ArcBytes<'a> {
+    fn as_ref(&self) -> &[u8] {
+        &**self
+    }
+}
+
 impl<'a> Read for ArcBytes<'a> {
     fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
         let end = self.buffer.len().min(self.position + buf.len());
@@ -572,6 +594,11 @@ impl<'a> Iterator for Iter<'a> {
             None
         }
     }
+
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        let length = self.buffer.len() - self.offset;
+        (length, Some(length))
+    }
 }
 
 #[test]
