@@ -67,6 +67,8 @@ impl Bytes {
     }
 }
 
+impl_std_cmp!(Bytes);
+
 impl<'a> std::fmt::Debug for Bytes {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let slice = self.as_slice();
@@ -88,6 +90,12 @@ impl<'a> From<&'a [u8]> for Bytes {
     }
 }
 
+impl<const N: usize> From<[u8; N]> for Bytes {
+    fn from(buffer: [u8; N]) -> Self {
+        Self(buffer.to_vec())
+    }
+}
+
 impl<'a> From<ArcBytes<'a>> for Bytes {
     fn from(buffer: ArcBytes<'a>) -> Self {
         Self(buffer.into_vec())
@@ -141,6 +149,8 @@ impl<'de> Deserialize<'de> for Bytes {
 #[derive(Clone, Eq, PartialEq, Ord, PartialOrd, Hash, Default)]
 pub struct CowBytes<'a>(pub Cow<'a, [u8]>);
 
+impl_std_cmp!(CowBytes<'a>);
+
 impl<'a> CowBytes<'a> {
     /// Returns the underlying Cow.
     #[allow(clippy::missing_const_for_fn)] // false positive
@@ -158,6 +168,12 @@ impl<'a> CowBytes<'a> {
             Cow::Owned(vec) => vec,
         }
     }
+
+    /// Returns a slice of the contained data.
+    #[must_use]
+    pub fn as_slice(&self) -> &[u8] {
+        &self.0
+    }
 }
 
 impl<'a> std::fmt::Debug for CowBytes<'a> {
@@ -211,6 +227,18 @@ impl<'a> From<&'a [u8]> for CowBytes<'a> {
     }
 }
 
+impl<'a, const N: usize> From<[u8; N]> for CowBytes<'a> {
+    fn from(buffer: [u8; N]) -> Self {
+        Self::from(buffer.to_vec())
+    }
+}
+
+impl<'a, const N: usize> From<&'a [u8; N]> for CowBytes<'a> {
+    fn from(buffer: &'a [u8; N]) -> Self {
+        Self(Cow::Borrowed(buffer))
+    }
+}
+
 impl<'a> Deref for CowBytes<'a> {
     type Target = Cow<'a, [u8]>;
 
